/// T-Rex : Copyright 2021 : Aldrin Mathew
///
/// AAF DEPENDABLE LICENSE - 1.0
///
/// This library is licensed under the AAF Dependable License 1.0.
/// You are allowed to use this library as a dependency in your
/// project(s) WITHOUT having to release/make available your own
/// project(s) under the same license.
///
/// Unless required by applicable law, this library is provided
/// "AS IS", WITHOUT ANY WARRANTIES OR CONDITIONS OF ANY KIND,
/// either express or implied. The author(s) of the project is
/// not liable for any harms, errors or troubles caused by using
/// this library, unless implied by law. By using this license,
/// you are acknowledging the complete terms and conditions of
/// licensing of this library as specified in AAF Dependable
/// License 1.0 available at this URL:
///
/// https://aldrinsartfactory.com/licenses/dependable/
///
/// You are NOT ALLOWED to sell, or distribute this
/// library under commercial or non-commercial purposes, other than
/// using it in your project. You are not allowed to revamp, build
/// upon, rebrand, refactor the contents of this library.
/// Your project shall contain "original work", ideally, the
/// definition of which is agreed upon by the community. Using
/// this library to achieve that is not a violation of this license.
/// You are NOT ALLOWED to use the name, branding and identity of
/// this library to identify or brand your own project. You ARE
/// however allowed to use the name and branding to pinpoint/show
/// the source of the contents/code/logic of your project. At no
/// point shall you use the identification of the Authors of this
/// library to associate them to your own projects, in a way that
/// is deceiving/misleading or gives out false information.
///

part of trex;

/// `Hookable` The immutable class that facilitates the use of Hooks in the simplest way possible by providing a new Widget implementation.
///
/// You must implement the `void hooks()` and the `Widget build(BuildContext)` methods so as to use this Widget.
///
/// ```dart
/// class MyCounter extends Hookable {
///   late final Hook<int> count;
///
///   MyCounter() {
///     count = Hook(0, this);
///   }
///
///   Widget build(_) {
///     return Scaffold(
///       body: Center(
///         child: Column(
///           mainAxisAlignment: MainAxisAlignment.center,
///           crossAxisAlignment: crossAxisAlignment.center,
///           children: [
///             Text(count.value.toString()),
///             IconButton(
///               onPressed: () {
///                 /// The widget automatically rebuilds when value of the Hook changes.
///                 count.value++;
///               }
///               icon: Icon(Icons.add),
///             ),
///           ]
///         ),
///       ),
///     );
///   }
/// }
/// ```
///
/// `hooks()` is provided as a way to organise all initialisation of hooks in your widget, and also facilitates the proper link between the Hook and the Hookable widget.
///
/// You may override the `void dispose()` method to do any cleanup before the widget gets unmounted from the widget tree.
@immutable
abstract class Hookable extends Widget {
  /// The `Hookable` constructor that passes the Key value to the `super` class to manage the widget during runtime.
  Hookable({Key? key}) : super(key: key);

  /// The variable that holds the StatelessElement of the corresponding widget after creation.
  late final HookableElement _hookableElement;

  /// Creates a HookableElement for the widget and stores it in the private Variable.
  /// WARNING: This function must not be overriden.
  @protected
  HookableElement createElement() {
    this._hookableElement = HookableElement(this);
    return this._hookableElement;
  }

  /// Represents the part of the Widget that defines the UI.
  /// This is called by the Framework when the widget is created and whenever the widget needs updating.
  @protected
  Widget build(BuildContext context);

  /// This method marks this widget to be rebuilt in the next frame. This is equivalent to `setState(() {});` in a `StatefulWidget`
  void _update() {
    try {
      this._hookableElement.markNeedsBuild();
    } catch (exception) {
      print(
        'TRex Exception: Problem encountered when marking the widget element for rebuild. Native message - ' +
            exception.toString(),
      );
    }
  }

  /// Implement this function and initialise all the hooks inside it.
  /// This function will automatically be called by the framework when the widget is constructed.
  ///
  /// ```
  /// ...
  /// late final Hook<int> count;
  /// void hooks() {
  ///   this.count = hook(0, this);
  /// }
  /// ...
  /// ```
  /// This is provided as a compact way to organise all initialization of the hooks inside your widget.
  void hooks();

  /// You can override this method to do any cleanup before the widget gets destroyed.
  ///
  /// This method will be automatically called by the framework.
  @protected
  void dispose() {}
}
